@@ -3,18 +3,10 @@ from ..helpers import Helpers
 
 
 class AuthHubConnection(BaseHubConnection):
-    def __init__(self, url, protocol, auth_function, keep_alive_interval=15, reconnection_handler=None,
-                 headers={}, verify_ssl=False, skip_negotiation=False):
+    def __init__(self, auth_function, headers={}, **kwargs):
         self.headers = headers
         self.auth_function = auth_function
-        super(AuthHubConnection, self).__init__(
-            url,
-            protocol,
-            headers=headers,
-            keep_alive_interval=keep_alive_interval,
-            reconnection_handler=reconnection_handler,
-            verify_ssl=verify_ssl,
-            skip_negotiation=skip_negotiation)
+        super(AuthHubConnection, self).__init__(**kwargs)
 
     def start(self):
         try:
@@ -24,7 +16,7 @@ class AuthHubConnection(BaseHubConnection):
             self.headers["Authorization"] = "Bearer " + self.token
             super(AuthHubConnection, self).start()
         except Exception as ex:
-            Helpers.get_logger().error(self.__class__.__name__)
-            Helpers.get_logger().error(str(ex))
+            Helpers.get_logger().warning(self.__class__.__name__)
+            Helpers.get_logger().warning(str(ex))
             raise ex
 
