@@ -1,8 +1,9 @@
 import websocket
 import threading
 import requests
-
+import traceback
 import uuid
+import logging
 import time
 import ssl
 from signalrcore.messages.message_type import MessageType
@@ -10,29 +11,11 @@ from signalrcore.messages.stream_invocation_message\
     import StreamInvocationMessage
 from .reconnection import ConnectionStateChecker
 from signalrcore.messages.ping_message import PingMessage
-from .connection_state import ConnectionState
+from .connection import ConnectionState
 from .errors import UnAuthorizedHubError, HubError
 from signalrcore.helpers import Helpers
-
-
-class StreamHandler(object):
-    def __init__(self, event, invocation_id):
-        self.event = event
-        self.invocation_id = invocation_id
-        self.next_callback = None
-        self.complete_callback = None
-        self.error_callback = None
-
-    def subscribe(self, subscribe_callbacks):
-        if subscribe_callbacks is None:
-            raise ValueError(" subscribe object must be {0}".format({
-                "next": None,
-                "complete": None,
-                "error": None
-                }))
-        self.next_callback = subscribe_callbacks["next"]
-        self.complete_callback = subscribe_callbacks["complete"]
-        self.error_callback = subscribe_callbacks["error"]
+from .handlers import StreamHandler, InvocationHandler
+from ..protocol.messagepack_protocol import MessagePackHubProtocol
 
 
 class BaseHubConnection(object):
@@ -44,7 +27,7 @@ class BaseHubConnection(object):
             keep_alive_interval=15,
             reconnection_handler=None,
             verify_ssl=False,
-            skip_negotiation=False):
+            skip_negotiation=False, **kwargs):
         self.skip_negotiation = skip_negotiation
         self.logger = Helpers.get_logger()
         self.url = url
@@ -64,8 +47,9 @@ class BaseHubConnection(object):
             keep_alive_interval
         )
         self.reconnection_handler = reconnection_handler
-        self.on_connect = None
-        self.on_disconnect = None
+        self.on_connect = lambda : self.logger.info("on_connect not defined")
+        self.on_disconnect = lambda : self.logger.info("on_disconnect not defined")
+        self.on_error = lambda error: self.logger.info("on_error not defined {0}".format(error))
 
     def negotiate(self):
         negotiate_url = Helpers.get_negotiate_url(self.url)
@@ -94,17 +78,20 @@ class BaseHubConnection(object):
     def start(self):
         if not self.skip_negotiation:
             self.negotiate()
+        
         self.logger.debug("Connection started")
+
         if self.state == ConnectionState.connected:
             self.logger.warning("Already connected unable to start")
-            return
+            return False
+
         self.state = ConnectionState.connecting
         self.logger.debug("start url:" + self.url)
         self._ws = websocket.WebSocketApp(
             self.url,
             header=self.headers,
             on_message=self.on_message,
-            on_error=self.on_error,
+            on_error=self.on_socket_error,
             on_close=self.on_close,
             on_open=self.on_open,
             )
@@ -114,12 +101,15 @@ class BaseHubConnection(object):
             ))
         self._thread.daemon = True
         self._thread.start()
+        
+        return True
 
     def stop(self):
         self.logger.debug("Connection stop")
         if self.state == ConnectionState.connected:
-            self._ws.close()
             self.connection_checker.stop()
+            self._ws.close()
+            #self._ws.teardown()           
             self.state = ConnectionState.disconnected
 
     def register_handler(self, event, callback):
@@ -150,9 +140,26 @@ class BaseHubConnection(object):
         if self.on_disconnect is not None and callable(self.on_disconnect):
             self.on_disconnect()
 
-    def on_error(self, error):
+    def on_socket_error(self, error):
+        """
+        Throws error related on https://github.com/websocket-client/websocket-client/issues/449
+
+        Args:
+            error ([type]): [description]
+
+        Raises:
+            HubError: [description]
+        """
         self.logger.debug("-- web socket error --")
-        self.logger.error("{0} {1}".format(error, type(error)))
+        if (type(error) is AttributeError and "'NoneType' object has no attribute 'connected'" in str(error)):
+            self.logger.warning("Websocket closing error: issue https://github.com/websocket-client/websocket-client/issues/449")
+            self.on_disconnect()                    
+        else:
+            self.logger.error(traceback.format_exc(5, True))
+            self.logger.error("{0} {1}".format(self, error))        
+            self.logger.error("{0} {1}".format(error, type(error)))
+            self.on_disconnect()
+            raise HubError(error)
 
     def on_message(self, raw_message):
         self.logger.debug("Message received{0}".format(raw_message))
@@ -190,10 +197,16 @@ class BaseHubConnection(object):
                 return
 
             if message.type == MessageType.completion:
+                if message.error is not None and len(message.error) > 0:
+                    self.on_error(message)
+
+                # Send callbacks
                 fired_handlers = list(
                     filter(
                         lambda h: h.invocation_id == message.invocation_id,
                         self.stream_handlers))
+
+                # Stream callbacks
                 for handler in fired_handlers:
                     handler.complete_callback(message)
 
@@ -237,10 +250,12 @@ class BaseHubConnection(object):
                         lambda h: h.invocation_id != message.invocation_id,
                         self.stream_handlers))
 
-    def send(self, message):
+    def send(self, message, on_invocation = None):
         self.logger.debug("Sending message {0}".format(message))
         try:
-            self._ws.send(self.protocol.encode(message))
+            if on_invocation:
+                self.stream_handlers.append(InvocationHandler(message.invocation_id, on_invocation))
+            self._ws.send(self.protocol.encode(message), opcode=0x2 if type(self.protocol) == MessagePackHubProtocol else 0x1)
             self.connection_checker.last_message = time.time()
             if self.reconnection_handler is not None:
                 self.reconnection_handler.reset()
@@ -270,14 +285,15 @@ class BaseHubConnection(object):
             threading.Thread(
                 target=self.deferred_reconnect,
                 args=(sleep_time,)
-            )
+            ).start()
 
     def deferred_reconnect(self, sleep_time):
         time.sleep(sleep_time)
         try:
             if not self.connection_alive:
-                self._send_ping()
+                self.send(PingMessage())
         except Exception as ex:
+            self.logger.error(ex)
             self.reconnection_handler.reconnecting = False
             self.connection_alive = False
 
@@ -287,8 +303,8 @@ class BaseHubConnection(object):
         self.stream_handlers.append(stream_obj)
         self.send(
             StreamInvocationMessage(
-                {},
                 invocation_id,
                 event,
-                event_params))
+                event_params,
+                headers=self.headers))
         return stream_obj
